/*
rsstool_write.c - write functions for RSStool

Copyright (c) 2006 NoisyB


This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#ifdef  HAVE_CONFIG_H
#include "config.h"
#endif
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>
#include <dirent.h>
#include <unistd.h>
#include <time.h>
#include "misc/itypes.h"
#include "misc/net.h"
#include "misc/string.h"
#include "misc/file.h"          // realpath2()
#include "misc/property.h"
#include "misc/rss.h"
#include "misc/sql.h"
#include "rsstool_defines.h"
#include "rsstool.h"
#include "rsstool_misc.h"


int
rsstool_write_property (st_rsstool_t *rt, const char *path)
{
  int items = rsstool_get_item_count (rt);
  char fname[FILENAME_MAX];
  char buf[MAXBUFSIZE];
  char value[64];
  FILE *fh = NULL;
  int i = 0;
  
  realpath2 (path, fname);

  if (!(fh = fopen (fname, "w")))
    return -1;

  fclose (fh);

  for (; i < items; i++)
    {
      sprintf (buf, "version_%d", i);
      set_property (fname, buf, rss_get_version_s_by_id (rt->item[i].version), !i ? "RSStool - RSS feed as property file" : NULL);

      sprintf (buf, "date_%d", i);
      sprintf (value, "%ld", rt->item[i].date);
      set_property (fname, buf, value, NULL);

      sprintf (buf, "url_%d", i);
      set_property (fname, buf, rt->item[i].url, NULL);

      sprintf (buf, "title_%d", i);
      set_property (fname, buf, rt->item[i].title, NULL);

      sprintf (buf, "desc_%d", i);
      set_property (fname, buf, rt->item[i].desc, NULL);
    }

  return 0;
}


static int
get_item_value (const char *tag)
{
  const char *p = net_tag_get_value (tag, "item");
  int value = p ? strtol (p, NULL, 10) : 0;

  return MAX (value, 0);
}


static const char *
template_url_filter (const char *s)
{
  return rsstool.item[get_item_value (s)].url;
}


static const char *
template_site_filter (const char *s)
{
  return rsstool.item[get_item_value (s)].site;
}


static const char *
template_date_filter (const char *s)
{
// TODO: seconds_only and other date layouts
  static char buf[MAXBUFSIZE];
  strftime (buf, MAXBUFSIZE, "%Y%m%d %H:%M", localtime (&rsstool.item[get_item_value (s)].date));
  return buf;
}


static const char *
template_title_filter (const char *s)
{
  return rsstool.item[get_item_value (s)].title;
}


static const char *
template_desc_filter (const char *s)
{
  return rsstool.item[get_item_value (s)].desc;
}


static const char *
template_start_filter (const char *s)
{
// TODO: make comment strings configurable in the parse tags
  if (get_item_value (s) >= rsstool.item_count)
    return "<!--";
  else
    return "";
}


static const char *
template_end_filter (const char *s)
{
  if (get_item_value (s) >= rsstool.item_count)
    return "-->";
  else
    return "";
}


static const char *
template_updated_filter (const char *s)
{
  (void) s;
  static char buf[MAXBUFSIZE];
  time_t t = time (0);

  strftime (buf, MAXBUFSIZE, "updated with RSStool " RSSTOOL_VERSION_S " %Y%m%d %H:%M:%S", localtime (&t));
  return buf;
}


static st_tag_filter_t f[] = {
  {
    "rsstool:url",
    template_url_filter
  },
  {
    "rsstool:title",
    template_title_filter
  },
  {
    "rsstool:desc",
    template_desc_filter
  },
  {
    "rsstool:date",
    template_date_filter
  },
  {
    "rsstool:site",
    template_site_filter
  },
  {
    "rsstool:start",
    template_start_filter
  },
  {
    "rsstool:end",
    template_end_filter
  },
  {
    "rsstool:updated",
    template_updated_filter
  },
  {NULL, NULL}
};


int
rsstool_write_template (st_rsstool_t *rt, const char *template)
{
  (void) rt;
  char fname[FILENAME_MAX];
  char buf[MAXBUFSIZE * 10];
  FILE *fh = NULL;
  int cf = 0;

  realpath2 (template, fname);

  if (!(fh = fopen (fname, "r")))
    return -1;

  while (fgets (buf, MAXBUFSIZE, fh))
    {
      cf = net_tag_filter (buf, f, cf);
      strrep (buf, "%", "%%");
      printf (buf);
    }

  fclose (fh);

  return 0;
}


int
rsstool_write_template_s (st_rsstool_t *rt, const char *template_s)
{
  (void) rt;
  char *p = (char *) malloc (strlen (template_s) * 2);

  if (p)
    {
      strcpy (p, template_s);
      net_tag_filter (p, f, 0);
      printf (p);
      free (p);

      return 0;
    }

  return -1;
}


int
rsstool_write_txt (st_rsstool_t *rt, int single_line)
{
  int items = rsstool_get_item_count (rt);
  int i = 0;

  for (; i < items; i++)
    {
      char buf[MAXBUFSIZE];
      strftime (buf, MAXBUFSIZE, "%Y%m%d %H:%M", localtime (&rt->item[i].date));

      if (single_line)
        printf ("%s %s %s %s\n",
          rt->item[i].site,
          buf,
          rt->item[i].url,
          rt->item[i].title);
      else
        printf ("%s %s\n"
                "(%s, %s)\n"
                "%s\n",
          rt->item[i].site,
          rt->item[i].title,
          rt->item[i].url,
          buf,
          rt->item[i].desc);
    }

  return 0;
}


int
rsstool_write_html (st_rsstool_t *rt, int single_line)
{
  int items = rsstool_get_item_count (rt);
  int i = 0;

  for (; i < items; i++)
    {
      char buf[MAXBUFSIZE];
      strftime (buf, MAXBUFSIZE, "%Y%m%d %H:%M", localtime (&rt->item[i].date));

      if (single_line)
        printf ("%s %s <a href=\"%s\">%s</a><br>\n",
          rt->item[i].site,
          buf,
          rt->item[i].url,
          rt->item[i].title);
      else
        printf ("%s <a href=\"%s\">%s</a><br>\n"
                "%s<br>\n"
                "%s<br>\n",
          rt->item[i].site,
          rt->item[i].url,
          rt->item[i].title,
          buf,
          rt->item[i].desc);
    }

  return 0;
}


int
rsstool_write_bookmarks (st_rsstool_t *rt)
{
  int items = rsstool_get_item_count (rt);
  int i = 0;

  printf ("<!DOCTYPE NETSCAPE-Bookmark-file-1>\n"
          "<META HTTP-EQUIV=\"Content-Type\" CONTENT=\"text/html; charset=UTF-8\">\n"
          "<TITLE>Bookmarks</TITLE>\n"
          "<H1>Bookmarks</H1>\n"
          "\n"
          "<DL><p>\n");

  for (; i < items; i++)
    printf ("    <DT><A HREF=\"%s\" ADD_DATE=\"%ld\">%s</A>\n", rt->item[i].url, (unsigned long) rt->item[i].date, rt->item[i].title);

  printf ("</DL><p>\n");

  return 0;
}


int
rsstool_write_csv (st_rsstool_t *rt)
{
  char buf[MAXBUFSIZE];
  int items = rsstool_get_item_count (rt);
  int i = 0;

  printf ("SITE,DATE,URL,TITLE,DESC\n");

  for (; i < items; i++)
    {
      sprintf (buf, "%s,%ld,%s,%s,%s\n",
        rt->item[i].site,
        (unsigned long) rt->item[i].date,
        rt->item[i].url,
        rt->item[i].title,
        rt->item[i].desc);
      printf (strcode (buf));
    }

  return 0;
}


int
rsstool_write_ansisql (st_rsstool_t *rt)
{
  int items = rsstool_get_item_count (rt);
  int i = 0;

  printf ("--------------------------------------------------------------\n"
          "-- RSStool - read, parse, merge and write RSS (and Atom) feeds\n"
          "--------------------------------------------------------------\n"
          "\n"
          "-- DROP TABLE IF EXISTS rsstool_table;\n"
          "-- CREATE TABLE IF NOT EXISTS rsstool_table\n"
          "-- (\n"
          "--   rsstool_site      longtext,\n"
          "--   rsstool_title     longtext,\n"
          "--   rsstool_url       longtext,\n"
          "--   rsstool_date      int(11),\n"
          "--   rsstool_desc      longtext\n"
          "-- );\n"
          "\n");

  for (; i < items; i++)
    {
      printf ("INSERT INTO rsstool_table (rsstool_site, rsstool_title, rsstool_url, rsstool_date, rsstool_desc) VALUES (\"%s\", \"%s\", \"%s\", %ld, \"%s\");\n",
        sql_escape_string (rt->item[i].site),
        sql_escape_string (rt->item[i].title),
        sql_escape_string (rt->item[i].url),
        rt->item[i].date,
        sql_escape_string (rt->item[i].desc));
    }

  return 0;
}


int
rsstool_write_rss (st_rsstool_t *rt, int version)
{
  st_rss_t rss;
  int i = 0;

  memset (&rss, 0, sizeof (st_rss_t));

  rss.version = version;
  strcpy (rss.title, "RSStool");
  strcpy (rss.url, "http://rsstool.berlios.de");
  strcpy (rss.desc, "read, parse, merge and write RSS (and Atom) feeds");

  for (; i < rsstool_get_item_count (rt); i++)
    {
      strncpy (rss.item[i].title, rt->item[i].title, RSSTOOL_MAXBUFSIZE)[RSSTOOL_MAXBUFSIZE - 1] = 0;
      strncpy (rss.item[i].url, rt->item[i].url, RSSTOOL_MAXBUFSIZE)[RSSTOOL_MAXBUFSIZE - 1] = 0;
      strncpy (rss.item[i].desc, rt->item[i].desc, RSSTOOL_MAXBUFSIZE)[RSSTOOL_MAXBUFSIZE - 1] = 0;
      rss.item[i].date = rt->item[i].date;
      rss.item_count++;
    }

  return rss_write (stdout, &rss, version);
}


#if 0
int
rsstool_write_mysql (st_rsstool_t *rt)
{
  st_sql_t *sql = NULL;
  st_strurl_t url;

  strurl (&url, rt->dburl);

  if (!(sql = sql_open (url.host, url.port, url.request, url.login, url.password, 0)))
    return -1;

  sql_ctrl (sql, "SELECT * FROM mysql.user");
  sql_ctrl (sql, "SELECT * FROM mysql.user");

  sql_close (sql);

  return 0;
}


int
rsstool_write_odbc (st_rsstool_t *rt)
{
  st_sql_t *sql = NULL;
  st_strurl_t url;

  strurl (&url, rt->dburl);

  if (!(sql = sql_open (url.host, url.port, url.request, url.login, url.password, 0)))
    return -1;

  sql_ctrl (sql, "SELECT * FROM mysql.user");
  sql_ctrl (sql, "SELECT * FROM mysql.user");

  sql_close (sql);

  return 0;
}
#endif
